import argparse
import logging
import os
import sys
from dotenv import load_dotenv
load_dotenv()
LOG_LEVEL = os.environ.get('LOG_LEVEL', 'INFO').upper()
CACHE_EXPIRY_DAYS = int(os.environ.get('CACHE_EXPIRY_DAYS', 7))

import logging
logging.basicConfig(
    format='%(asctime)s %(levelname)s: %(message)s',
    datefmt='%Y-%m-%d %H:%M:%S',
    level=LOG_LEVEL
)

from helpers.PlaylistBuilder import PlaylistBuilder
from helpers.ScoreSaberAPI import ScoreSaberAPI
from helpers.BeatLeaderAPI import BeatLeaderAPI
from helpers.SimpleBeatLeaderAPI import SimpleBeatLeaderAPI
from helpers.SimpleBeatSaverAPI import SimpleBeatSaverAPI

from saberlist.utils import reset_history
from saberlist.playlist_strategies.oldscores import (
    playlist_strategy_beatleader_oldscores,
    playlist_strategy_scoresaber_oldscores,
)
from saberlist.playlist_strategies.accuracy import (
    playlist_strategy_beatleader_lowest_acc,
    playlist_strategy_beatleader_accuracy_gaps,
    playlist_strategy_scoresaber_accuracy_gaps,
    playlist_strategy_beatleader_accuracy_gaps_star_range,
)
from saberlist.playlist_strategies.performance import (
    playlist_strategy_beatleader_lowest_pp,
    playlist_strategy_scoresaber_lowest_pp,
)
from saberlist.playlist_strategies.beatsaver import (
    playlist_strategy_beatsaver_acc,
    playlist_strategy_beatsaver_curated,
    playlist_strategy_beatsaver_mappers,
)

def saberlist() -> None:
    """
    Generate a playlist of songs using a specified strategy.
    Avoids reusing the same song+difficulty in a playlist based on history.
    """
    strategy = get_strategy()

    if strategy == 'scoresaber_oldscores':
        playlist_data, playlist_title = playlist_strategy_scoresaber_oldscores(ScoreSaberAPI(cache_expiry_days=CACHE_EXPIRY_DAYS))
        playlist_builder = PlaylistBuilder()
    elif strategy == 'beatleader_oldscores':
        playlist_data, playlist_title = playlist_strategy_beatleader_oldscores(BeatLeaderAPI(cache_expiry_days=CACHE_EXPIRY_DAYS))
        playlist_builder = PlaylistBuilder()
    elif strategy == 'beatsaver_acc':
        playlist_data, playlist_title = playlist_strategy_beatsaver_acc()
        playlist_builder = PlaylistBuilder(covers_dir='./covers/beatsavers')
    elif strategy == 'beatleader_lowest_pp':
        playlist_data, playlist_title = playlist_strategy_beatleader_lowest_pp(BeatLeaderAPI(cache_expiry_days=CACHE_EXPIRY_DAYS))
        playlist_builder = PlaylistBuilder(covers_dir='./covers/pajamas')
    elif strategy == 'scoresaber_lowest_pp':
        playlist_data, playlist_title = playlist_strategy_scoresaber_lowest_pp(ScoreSaberAPI(cache_expiry_days=CACHE_EXPIRY_DAYS))
        playlist_builder = PlaylistBuilder(covers_dir='./covers/scoresaber')
    elif strategy == 'beatleader_lowest_acc':
        playlist_data, playlist_title = playlist_strategy_beatleader_lowest_acc(BeatLeaderAPI(cache_expiry_days=CACHE_EXPIRY_DAYS))
        playlist_builder = PlaylistBuilder(covers_dir='./covers/kaiju')
    elif strategy == 'beatleader_accuracy_gaps':
        playlist_data, playlist_title = playlist_strategy_beatleader_accuracy_gaps(SimpleBeatLeaderAPI(cache_expiry_days=CACHE_EXPIRY_DAYS))
        playlist_builder = PlaylistBuilder(covers_dir='./covers/pajamas')
    elif strategy == 'beatleader_accuracy_gaps_star_range':
        input_star_level = input("Enter star level (Default: 6)") or 6
        playlist_data, playlist_title = playlist_strategy_beatleader_accuracy_gaps_star_range(SimpleBeatLeaderAPI(cache_expiry_days=CACHE_EXPIRY_DAYS), star_level=float(input_star_level))
        playlist_builder = PlaylistBuilder(covers_dir='./covers/pajamas')
    elif strategy == 'scoresaber_accuracy_gaps':
        playlist_data, playlist_title = playlist_strategy_scoresaber_accuracy_gaps(ScoreSaberAPI(cache_expiry_days=CACHE_EXPIRY_DAYS))
        playlist_builder = PlaylistBuilder(covers_dir='./covers/scoresaber')
    elif strategy == 'beatsaver_curated':
        playlist_data, playlist_title = playlist_strategy_beatsaver_curated(SimpleBeatSaverAPI())
        playlist_builder = PlaylistBuilder(covers_dir='./covers/curated')
    elif strategy == 'beatsaver_mappers':
        playlist_data, playlist_title = playlist_strategy_beatsaver_mappers(SimpleBeatSaverAPI())
        playlist_builder = PlaylistBuilder(covers_dir='./covers/pajamas')
    elif strategy == 'blank_playlist':
        playlist_data, playlist_title = [], input("Enter playlist title: ")
        playlist_builder = PlaylistBuilder(covers_dir='./covers/pajamas')
    else:
        logging.error(f"Unknown strategy '{strategy}'")
        return

    if not playlist_data and strategy != 'blank_playlist':
        logging.info("No new scores found to add to the playlist.")
        return

    playlist_builder.create_playlist(
        playlist_data,
        playlist_title=playlist_title,
        playlist_author="SaberList Tool"
    )

def get_strategy():
    parser = argparse.ArgumentParser(description="Generate Beat Saber playlists")
    parser.add_argument("-s", "--strategy", 
                        choices=[
                            "scoresaber_oldscores",
                            "beatleader_oldscores",
                        #   "beatsaver_acc", 
                        #   "beatleader_lowest_pp",
                        #   "scoresaber_lowest_pp",
                        #   "beatleader_lowest_acc",
                            "beatleader_accuracy_gaps",
                            "beatleader_accuracy_gaps_star_range",
                            "scoresaber_accuracy_gaps",
                            "beatsaver_curated",
                            "beatsaver_mappers",
                            "blank_playlist"
                        ],
                        help="Specify the playlist generation strategy")
    parser.add_argument("-r", "--reset",
                        action="store_true",
                        help="Reset the history for the specified strategy")
    
    if len(sys.argv) == 1:
        parser.print_help()
        sys.exit(1)
    
    args = parser.parse_args()
    
    if args.reset:
        if not args.strategy:
            parser.error("--reset requires --strategy to be specified")
        reset_history(args.strategy)
        sys.exit(0)
    
    if not args.strategy:
        parser.error("--strategy is required unless --reset is used")
    
    return args.strategy
