from datetime import timedelta
from typing import Dict, Any, List
import json
import logging
import os
from dotenv import load_dotenv
load_dotenv()
LOG_LEVEL = os.environ.get('LOG_LEVEL', 'INFO').upper()
HISTORY_FILE = os.environ.get('HISTORY_FILE', "playlist_history.json")

import logging
logging.basicConfig(
    format='%(asctime)s %(levelname)s: %(message)s',
    datefmt='%Y-%m-%d %H:%M:%S',
    level=LOG_LEVEL
)

import calendar

def load_history() -> Dict[str, Any]:
    """
    Load the playlist history from a JSON file.
    
    :return: A dictionary containing the history.
    """
    if os.path.exists(HISTORY_FILE):
        with open(HISTORY_FILE, 'r') as f:
            history = json.load(f)
            history.setdefault('playlist_counts', {})
            return history
    return {'scoresaver': {}, 'playlist_counts': {}}

def save_history(history: Dict[str, Any]) -> None:
    """
    Save the playlist history to a JSON file.
    
    :param history: The history dictionary to save.
    """
    with open(HISTORY_FILE, 'w') as f:
        json.dump(history, f, indent=2)

def prompt_for_player_id(default_id: str = '76561199407393962') -> str:
    """
    Prompt the user to enter a ScoreSaber or BeatLeader player ID.
    Uses a default ID if the user presses Enter without input.
    
    :param default_id: The default player ID to use.
    :return: The player ID entered by the user or the default.
    """
    prompt = f"Enter player ID (press Enter for default '{default_id}'): "
    player_id = input(prompt).strip() or default_id
    return player_id

def prompt_for_mapper_ids() -> List[int]:
    default_mapper_ids = [
        4285547, # Avexus
        4330286, # VoltageO
        4294118, # Spinvvy
        4284542, # PogU (ForsenCDPogU)
        4285738, # Lekrkoekj
        113133   # Cush
    ]
    prompt = f"Enter mapper IDs (Default: {default_mapper_ids}): "
    mapper_ids = input(prompt).strip() or ",".join(map(str, default_mapper_ids))
    return [int(id) for id in mapper_ids.split(',')]

def format_time_ago(time_difference: timedelta) -> str:
    """
    Format a timedelta object into a human-readable string.
    
    :param time_difference: The time difference to format.
    :return: A string representing the time difference (e.g., '5d', '2w').
    """
    days = time_difference.days
    
    if days < 7:
        return f"{days} day(s)"
    elif days < 30:
        weeks = days // 7
        return f"{weeks} week(s)"
    elif days < 365:
        months = days // 30
        return f"{months} month(s)"
    else:
        years = days // 365
        return f"{years} year(s)"

def normalize_difficulty_name(difficulty_name):
    difficulty_names = {
        # ScoreSaber
        '_ExpertPlus_SoloStandard': 'expertplus',
        '_Expert_SoloStandard': 'expert',
        '_Hard_SoloStandard': 'hard',
        '_Normal_SoloStandard': 'normal',
        '_Easy_SoloStandard': 'easy',
        # BeatLeader
        1: 'easy',
        3: 'normal',
        5: 'hard',
        7: 'expert',
        9: 'expertplus',
    }
    
    # Return the mapped value or the original name if there is no mapping
    return difficulty_names.get(difficulty_name, difficulty_name)

"""deprecated in favor of using undocumented api call
def infer_beatleader_leaderboard_id(song_id: str, difficulty: str) -> str:
    difficulty_map = {
        'Easy': 1,
        'Normal': 3,
        'Hard': 5,
        'Expert': 7,
        'ExpertPlus': 9,
    }
    return f"{song_id}{difficulty_map[difficulty]}1"
"""

def reset_history(strategy: str) -> None:
    """
    Reset the history for a given playlist strategy.
    
    :param strategy: The strategy to reset history for.
    """
    history = load_history()
    if strategy in history:
        del history[strategy]
        if 'playlist_counts' in history and strategy in history['playlist_counts']:
            history['playlist_counts'][strategy] = 0
        save_history(history)
        logging.info(f"History and playlist count for '{strategy}' have been reset.")
    else:
        logging.info(f"No history found for '{strategy}'. Nothing to reset.")
